open Ast

(* this module is responsible for transforming the AST into a sequence of
   display event markers (set property / display text) *)
 
type prop = { is_root : bool;
	      b  : bool; em : bool;
	      i  : bool; s  : bool;
	      tt : bool; u  : int;
	      size : string; 
	      color : string }

let plain = {b=false; em=false; i=false; 
	     s=false; tt=false; u=0; 
	     size="12"; color="b"}

let is_plain p = (p.u = 0 && not (p.s || p.em || p.i || p.b || p.tt))      

type prop_chg = SetB of bool
		| SetEm of bool
		| SetI of bool
		| SetS of bool
		| SetTT of bool
		| SetPlain of bool
		| SetU of int
		| SetSz of string
		| SetCol of string

type event = SetProps of (prop_chg list)
	     | Text of string

let (doc_events : event list ref) = ref []

let set_bold p = if p.b then None else Some ({p with b = true},SetB true. SetB false)
let set_em p = if p.s then None else Some ({p with em = (not p.em)},SetEm true, SetEm false)
let set_i p = if p.i then None else Some ({p with i = true},SetI true, SetI false)
let set_s p = if p.s && (not p.em) then None else Some ({p with s=true; em=false}, SetS true, SetS false) 
let set_tt p = if p.tt then None else Some ({p with tt=true}, SetTT true, SetTT false)
let set_u p = if p.u < 3 then Some ({p with u=(p.u+1)}, SetU (p.u+1), SetU p.u) else None

let set_pl p = (if is_plain p then None 
		else Some ({plain with size=p.size; color=p.color}, SetPlain true, SetPlain false))

let set_size x p = (if x = p.size then None else Some ({p with size=x}, SetSz x, SetSz p.size))
let set_col c p = (if c = p.color then None else Some ({p with color=c}, SetCol c, SetCol p.color))
		    
let rec adjust oldprops setter doc = 
  match setter oldprops with
      None -> canonicalize oldprops doc
    | Some (newprops,propchg,propreset) ->
	begin    
	  doc_events := ((SetProps propchg) :: (!doc_events));
	  canonicalize (newprops,newpropchgs) doc;
	  doc_events := ((SetProps propreset) :: (!doc_events))
	end
	  
and canonicalize props d = match d with
    Seq sz -> List.iter (canonicalize props) sz
  | Txt s -> doc_events := ((Text s) :: (!doc_events))
  | B d  -> adjust props set_bold d
  | EM d -> adjust props set_em d
  | I d  -> adjust props set_i d
  | PL d -> adjust props set_pl d
  | S d  -> adjust props set_s d
  | TT d -> adjust props set_tt d
  | U d  -> adjust props set_u d
  | Size (x,d) -> adjust props (set_size x) d
  | Col (c,d)  -> adjust props (set_col c) d
  | Empty -> ()

